#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <d3d8.h>
#include "dbe.h"

#include "d3dconfig.h"

/* Dialog box definitions */
#define MAVD3D_ITEMS 13

#define MAVD3D_ADAPTERTEXT 10
#define MAVD3D_ADAPTER 11
#define MAVD3D_DEVICETEXT 12
#define MAVD3D_DEVICE 13
#define MAVD3D_DMTEXT 14
#define MAVD3D_DM 15
#define MAVD3D_FULLSCREEN 16
#define MAVD3D_TDGROUP 17
#define MAVD3D_TD16 18
#define MAVD3D_TD32 19
#define MAVD3D_ALPHA 20
#define MAVD3D_CANCEL 21
#define MAVD3D_SAVE 22

/* array containing dialog item data */
DBE_DIALOG_ITEM mavd3d_list[MAVD3D_ITEMS]= {
  {0, WS_VISIBLE|WS_CHILD|SS_RIGHT, 4, 5, 38, 8, MAVD3D_ADAPTERTEXT, 0x0082, "Display adapter", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|CBS_DROPDOWNLIST|CBS_HASSTRINGS, 50, 4, 120, 80, MAVD3D_ADAPTER, 0x0085, "", 1},
  {0, WS_VISIBLE|WS_CHILD|SS_RIGHT, 4, 21, 38, 8, MAVD3D_DEVICETEXT, 0x0082, "D3D device", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|CBS_DROPDOWNLIST|CBS_HASSTRINGS, 50, 20, 120, 80, MAVD3D_DEVICE, 0x0085, "", 1},
  {0, WS_VISIBLE|WS_CHILD|SS_RIGHT, 4, 37, 38, 8, MAVD3D_DMTEXT, 0x0082, "Display mode", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|WS_VSCROLL|CBS_DROPDOWNLIST|CBS_HASSTRINGS, 50, 36, 70, 80, MAVD3D_DM, 0x0085, "", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|BS_CHECKBOX, 74, 52, 36, 8, MAVD3D_FULLSCREEN, 0x0080, "Fullscreen", 1},
  {0, WS_VISIBLE|WS_CHILD|BS_GROUPBOX, 13, 51, 46, 20, MAVD3D_TDGROUP, 0x0080, "Texture depth", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|BS_RADIOBUTTON, 18, 60, 17, 8, MAVD3D_TD16, 0x0080, "16", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|BS_RADIOBUTTON, 40, 60, 17, 8, MAVD3D_TD32, 0x0080, "32", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|BS_CHECKBOX, 74, 64, 35, 8, MAVD3D_ALPHA, 0x0080, "1 bit alpha", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|BS_PUSHBUTTON, 136, 36, 34, 16, MAVD3D_CANCEL, 0x0080, "Cancel", 1},
  {0, WS_VISIBLE|WS_CHILD|WS_TABSTOP|BS_DEFPUSHBUTTON, 136, 56, 34, 16, MAVD3D_SAVE, 0x0080, "Save", 1}
};

/* globals */
short mavd3d_x= 477;
short mavd3d_y= 135;
short mavd3d_cx= 175;
short mavd3d_cy= 77;
char mavd3d_txt[]= {"Maverik D3D configuration"};


/* current device selections */
int current_adapter= 0;
int current_device= 0;

/* current option choices */
int d3d_fullscreen= 1;
int d3d_texturesize= 16;
int d3d_alpha= 0;

/* display modes */
int mode_list[100];
int mode_i= 0;
int curr_dm= 0;


void device_selected (HWND hwnd, int sel)
{
/* a D3D device has been selected */
  int i;
  char buff[1000];

  current_device= sel;

/* 16 bit textures */
  EnableWindow (GetDlgItem (hwnd, MAVD3D_TD16), 1);

/* 32 bit textures */
  EnableWindow (GetDlgItem (hwnd, MAVD3D_TD32), 1);

/* 1 bit alpha textures */
  if (d3d_texturesize==16)
    EnableWindow (GetDlgItem (hwnd, MAVD3D_ALPHA), 1);
  else
    EnableWindow (GetDlgItem (hwnd, MAVD3D_ALPHA), 0);

/* compatible display modes */
  SendMessage (GetDlgItem (hwnd, MAVD3D_DM), CB_RESETCONTENT, 0, 0);

  mode_i= 0;
  for (i=0; i<adapter_list[current_adapter].dmode_count; i++) {
    if (adapter_list[current_adapter].device_dmodes[current_device][i]) {
      sprintf (buff, "%ldx%ldx%ldbbp",
		adapter_list[current_adapter].dmodes[i].width,
		adapter_list[current_adapter].dmodes[i].height,
		adapter_list[current_adapter].dmodes[i].depth);
/* need to use mode_i rather than i because combo boxes need entries with */
/* consecutive IDs */
      SendMessage (GetDlgItem (hwnd, MAVD3D_DM), CB_INSERTSTRING, mode_i,
		(LPARAM) buff);
      mode_list[mode_i]= i;

      if (curr_dm==i) {
/* keep current display mode selection if it is available */
	SendMessage (GetDlgItem (hwnd, MAVD3D_DM), CB_SETCURSEL,
		(WPARAM) mode_i, 0);
	curr_dm= i;
      }
      mode_i ++;
    }
  }
}

void adapter_selected (HWND hwnd, int sel)
{
/* a display adapter has been selected */
  current_adapter= sel;

/* set D3D devices */
  SendMessage (GetDlgItem (hwnd, MAVD3D_DEVICE), CB_RESETCONTENT, 0, 0);
  SendMessage (GetDlgItem (hwnd, MAVD3D_DEVICE), CB_INSERTSTRING, 0,
		(LPARAM) adapter_list[current_adapter].device_list[0].name);
  SendMessage (GetDlgItem (hwnd, MAVD3D_DEVICE), CB_INSERTSTRING, 1,
		(LPARAM) adapter_list[current_adapter].device_list[1].name);

  SendMessage (GetDlgItem (hwnd, MAVD3D_DEVICE), CB_SETCURSEL,
		current_device, 0);
  device_selected (hwnd, current_device);
}

void d3d_save (void)
{
  FILE *fp;
  char filename[256];

  sprintf (filename, "%s/.mavd3d8_config", getenv("HOME"));

  fp= fopen (filename, "w");

  if (fp) {
    fprintf (fp, "%s\n", adapter_list[current_adapter].name);
    fprintf (fp, "%d %ld %ld %ld %d\n",
	adapter_list[current_adapter].device_list[current_device].id,
	adapter_list[current_adapter].dmodes[curr_dm].width,
	adapter_list[current_adapter].dmodes[curr_dm].height,
	adapter_list[current_adapter].dmodes[curr_dm].depth,
	adapter_list[current_adapter].dmodes[curr_dm].format);
    fprintf (fp, "%d\n", d3d_texturesize);
    fprintf (fp, "%d %d\n", d3d_alpha, d3d_fullscreen);
    fprintf (fp, "%d %d\n", mavd3d_x, mavd3d_y);

    fclose (fp);
  } else {
    char buff[256];

    sprintf (buff, "Couldn't open file: %s", filename);
    MessageBox (NULL, buff, NULL, MB_OK);
  }
}

void readline (FILE *fp, char *str)
{
  char ch;
  int i= 0;

/* scan in chars until end of line */
  ch= getc (fp);

  while (ch != '\n') {
    str[i]= ch;
    i ++;
    ch= getc (fp);
  }

  str[i]=(char) NULL;
}

void d3d_load (void)
{
  char filename[256];
  FILE *fp;

  sprintf (filename, "%s/.mavd3d8_config", getenv("HOME"));

  fp= fopen (filename, "r");

  if (fp) {
    char d3d_name[100];
    int i;
    int dev,w,h,d,f;

    readline (fp, d3d_name);
    for (i=0; i<adapter_count; i++) {
      if (!strcmp (adapter_list[i].name, d3d_name))
	current_adapter= i;
    }

    fscanf (fp, "%d %d %d %d %d", &dev, &w, &h, &d, &f);
    if (adapter_list[current_adapter].device_list[i].id==dev)
      current_device= i;

    for (i=0; i<adapter_list[current_adapter].dmode_count; i++) {
      if (adapter_list[current_adapter].dmodes[i].width==w &&
		adapter_list[current_adapter].dmodes[i].height==h &&
		adapter_list[current_adapter].dmodes[i].depth==d)
	curr_dm= i;
    }

    fscanf (fp, "%d %d %d", &d3d_texturesize, &d3d_alpha,
			&d3d_fullscreen);

    fscanf (fp, "%d %d", &w, &h);

    mavd3d_x= w;
    mavd3d_y= h;

    fclose (fp);
  }
/* if no file just use defaults */
}

/* Dialog event handler */
BOOL CALLBACK mavd3d_event (HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam)
{
  int i;

/* check message type */
  switch (umsg) {
    case WM_INITDIALOG:
      dbe_ansi_font (hwnd, MAVD3D_ITEMS, mavd3d_list);

/* empty combo boxes */
      SendMessage (GetDlgItem (hwnd, MAVD3D_DEVICE), CB_RESETCONTENT, 0, 0);
      SendMessage (GetDlgItem (hwnd, MAVD3D_ADAPTER), CB_RESETCONTENT, 0, 0);
      SendMessage (GetDlgItem (hwnd, MAVD3D_DM), CB_RESETCONTENT, 0, 0);

/* set display adapters */
      for (i=0; i<adapter_count; i++) {
	SendMessage (GetDlgItem (hwnd, MAVD3D_ADAPTER), CB_INSERTSTRING, i,
		(LPARAM) adapter_list[i].desc);
      }

/* set fullscreen */
      if (d3d_fullscreen)
	CheckDlgButton (hwnd, MAVD3D_FULLSCREEN, BST_CHECKED);
      else
	CheckDlgButton (hwnd, MAVD3D_FULLSCREEN, BST_UNCHECKED);

/* set 1 bit alpha */
      if (d3d_alpha)
	CheckDlgButton (hwnd, MAVD3D_ALPHA, BST_CHECKED);
      else
	CheckDlgButton (hwnd, MAVD3D_ALPHA, BST_UNCHECKED);

/* set texture depth */
      if (d3d_texturesize==16) {
	EnableWindow (GetDlgItem (hwnd, MAVD3D_ALPHA), 1);
	CheckRadioButton (hwnd, MAVD3D_TD16, MAVD3D_TD32, MAVD3D_TD16);
      } else {
	CheckRadioButton (hwnd, MAVD3D_TD16, MAVD3D_TD32, MAVD3D_TD32);
      }

/* set current driver choices */
      SendMessage (GetDlgItem (hwnd, MAVD3D_ADAPTER), CB_SETCURSEL,
			current_adapter, 0);
      adapter_selected (hwnd, current_adapter);
      return 1;
    case WM_MOVE:
      mavd3d_x= LOWORD (lparam)/2;
      mavd3d_y= HIWORD (lparam)/2;
      return 1;
    case WM_QUIT:
    case WM_DESTROY:
      EndDialog (hwnd, 0);
      return 1;
    case WM_COMMAND:
      switch (LOWORD (wparam)) {
        case IDCANCEL:
	case MAVD3D_CANCEL:
          EndDialog (hwnd, 0);
          return 1;
	case MAVD3D_SAVE:
	  d3d_save ();
	  EndDialog (hwnd, 0);
	  return 1;
	case MAVD3D_TD16:
	  d3d_texturesize= 16;
	  CheckRadioButton (hwnd, MAVD3D_TD16, MAVD3D_TD32, MAVD3D_TD16);
	  EnableWindow (GetDlgItem (hwnd, MAVD3D_ALPHA), 1);
	  return 1;
	case MAVD3D_TD32:
	  d3d_texturesize= 32;
	  CheckRadioButton (hwnd, MAVD3D_TD16, MAVD3D_TD32, MAVD3D_TD32);
	  EnableWindow (GetDlgItem (hwnd, MAVD3D_ALPHA), 0);
	  return 1;
	case MAVD3D_ALPHA:
	  d3d_alpha= !d3d_alpha;
	  if (d3d_alpha)
	    CheckDlgButton (hwnd, MAVD3D_ALPHA, BST_CHECKED);
	  else
	    CheckDlgButton (hwnd, MAVD3D_ALPHA, BST_UNCHECKED);
	  return 1;
	case MAVD3D_FULLSCREEN:
	  d3d_fullscreen= !d3d_fullscreen;
	  if (d3d_fullscreen)
	    CheckDlgButton (hwnd, MAVD3D_FULLSCREEN, BST_CHECKED);
	  else
	    CheckDlgButton (hwnd, MAVD3D_FULLSCREEN, BST_UNCHECKED);
	  return 1;
	case MAVD3D_ADAPTER:
	  if (HIWORD(wparam)==CBN_SELCHANGE) {
	    int sel;

	    sel= SendMessage (GetDlgItem (hwnd, MAVD3D_ADAPTER),
			CB_GETCURSEL, 0, 0);
	    if (sel != CB_ERR && sel != current_adapter) {
	      adapter_selected (hwnd, sel);
	      return 1;
	    }
	  }
	  return 0;
	case MAVD3D_DEVICE:
	  if (HIWORD(wparam)==CBN_SELCHANGE) {
	    int sel;

	    sel= SendMessage (GetDlgItem (hwnd, MAVD3D_DEVICE),
			CB_GETCURSEL, 0, 0);
	    if (sel != CB_ERR && sel != current_device) {
	      device_selected (hwnd, sel);
	      return 1;
	    }
	  }
	  return 0;
	case MAVD3D_DM:
	  if (HIWORD(wparam)==CBN_SELCHANGE) {
	    int sel;

	    sel= SendMessage (GetDlgItem (hwnd, MAVD3D_DM),
			CB_GETCURSEL, 0, 0);
	    if (sel != CB_ERR) {
	      curr_dm= mode_list[sel];
	      return 1;
	    }
	  }
	  return 0;
        default:
          return 0;
      }
    default:
      return 0;
  }
}

/* main dialog function */
void d3dconfigbox (void)
{
  void *mavd3d_malloc;

/* read in current settings */
  d3d_load ();

/* create dialog template */
  mavd3d_malloc= dbe_dialog_create (MAVD3D_ITEMS,
        mavd3d_x-1, mavd3d_y-9, mavd3d_cx, mavd3d_cy,
        mavd3d_txt, mavd3d_list);

/* create and run dialog box */
  DialogBoxIndirect (GetModuleHandle (NULL),
        dbe_next_DWORD (mavd3d_malloc), NULL, mavd3d_event);

/* release memory */
  dbe_free (mavd3d_malloc);
}


