#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <d3d8.h>

#include "d3dconfig.h"

LPDIRECT3D8 D3D;

/* enumerated info */
AdapterInfo adapter_list[10];
int adapter_count= 0;

/* typedef for create function */
typedef struct IDirect3D8 * WINAPI (*d3dfunctype) (UINT);

void get_adapter_info (int id)
{
  D3DADAPTER_IDENTIFIER8 info;
  D3DDISPLAYMODE dm;
  int i, prev;
  int dm_ok;

  D3D->lpVtbl->GetAdapterIdentifier (D3D, id, D3DENUM_NO_WHQL_LEVEL, &info);

/* store the name and description of the adapter */
  sprintf (adapter_list[id].name, "%s", info.Driver);
  sprintf (adapter_list[id].desc, "%s", info.Description);

/* get display modes */
  adapter_list[id].dmode_count= D3D->lpVtbl->GetAdapterModeCount (D3D, id);

  for (i=0; i<adapter_list[id].dmode_count; i++) {
    D3D->lpVtbl->EnumAdapterModes (D3D, id, i, &dm);
/* check this is a new mode (ignore duplications due to refresh rates) */
    dm_ok= 1;
    for (prev= 0; prev<i; prev++) {
      if (adapter_list[id].dmodes[prev].format==dm.Format &&
	  adapter_list[id].dmodes[prev].width==dm.Width &&
	  adapter_list[id].dmodes[prev].height==dm.Height)
	dm_ok= 0;
    }

    if (dm_ok) {
/* store display modes */
      if (D3D->lpVtbl->CheckDeviceFormat (D3D, id, D3DDEVTYPE_HAL, dm.Format,
		D3DUSAGE_RENDERTARGET, D3DRTYPE_SURFACE, dm.Format)==D3D_OK)
        adapter_list[id].device_dmodes[DEVICE_HAL][i]= 1;
      else
        adapter_list[id].device_dmodes[DEVICE_HAL][i]= 0;

      if (D3D->lpVtbl->CheckDeviceFormat (D3D, id, D3DDEVTYPE_REF, dm.Format,
		D3DUSAGE_RENDERTARGET, D3DRTYPE_SURFACE, dm.Format)==D3D_OK)
        adapter_list[id].device_dmodes[DEVICE_REF][i]= 1;
      else
        adapter_list[id].device_dmodes[DEVICE_REF][i]= 0;

      adapter_list[id].dmodes[i].format= dm.Format;
      adapter_list[id].dmodes[i].width= dm.Width;
      adapter_list[id].dmodes[i].height= dm.Height;
      if (dm.Format==D3DFMT_X8R8G8B8 || dm.Format==D3DFMT_A8R8G8B8)
        adapter_list[id].dmodes[i].depth= 32;
      else
	adapter_list[id].dmodes[i].depth= 16;
    }
  }

  sprintf (adapter_list[id].device_list[DEVICE_HAL].name,
		"D3D Hardware abstraction layer");
  adapter_list[id].device_list[DEVICE_HAL].id= D3DDEVTYPE_HAL;

  sprintf (adapter_list[id].device_list[DEVICE_REF].name,
		"D3D Software reference device");
  adapter_list[id].device_list[DEVICE_REF].id= D3DDEVTYPE_REF;
}

void enumfunc (void)
{
  HINSTANCE d3dlib;
  d3dfunctype d3dfunc;
  int i;

/* get D3D8 interface */
  d3dlib= LoadLibrary ("D3D8.DLL");
  if (!d3dlib) {
    fprintf (stderr, "Error: Load library (d3d8.dll) failed\n");
    exit (1);
  }

/* get create function */
  d3dfunc= (d3dfunctype) GetProcAddress (d3dlib, "Direct3DCreate8");

  if (!d3dfunc) {
    fprintf (stderr, "Error: Get function (Direct3DCreate8) failed\n");
    exit (1);
  }

/* create d3d handle */
  D3D= (*d3dfunc) (D3D_SDK_VERSION);

/* enumerate d3d adapters */
  adapter_count= D3D->lpVtbl->GetAdapterCount (D3D);

  for (i=0; i<adapter_count; i++) {
    get_adapter_info (i);
  }

/* clean up */
  D3D->lpVtbl->Release (D3D);
  FreeLibrary (d3dlib);
}

int main (int argc, char *argv[])
{
/* enumerate devices */
  enumfunc ();

/* create dialog box */
  d3dconfigbox ();

  return 1;
}
 
